/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Alexander Bokovoy <alexander.bokovoy@nokia.com>
**
** This file is part of the Quill Image Filters package.
**
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/

/*!
  \class RedEyeReduction

  \brief Filter to remove red eyes detected by RedEyeDetection.

Options supported: SelectionArea (polygon approximating the eye area
in full-image coordinates).

Default behavior: none.
*/

#ifndef __QUILL_FILTERS_RED_EYE_REDUCTION_H__
#define __QUILL_FILTERS_RED_EYE_REDUCTION_H__

#include "quillimagefilterimplementation.h"

class RedEyeReductionPrivate;

class RedEyeReduction : public QuillImageFilterImplementation
{
public:
    RedEyeReduction();
    ~RedEyeReduction();

    QuillImage apply(const QuillImage &image) const;

    bool setOption(const QString &option, const QVariant &value);
    QVariant option(const QString &option) const;
    const QStringList supportedOptions() const;

    virtual const QString name() const { return QuillImageFilter::Name_RedEyeReduction; }

private:
    RedEyeReductionPrivate *priv;
    int compare(double realValue, double minimum, double maximum) const;
    void generatePolygon() const;

    void fractionToFullImageCoordinates (const QuillImage &image,
					const QPoint &frac,
					QPoint &full) const;
    void fractionToFullImageRect (const QuillImage &image,
				 const QRect &frac,
				 QRect &full) const;
    void fullImageToFractionCoordinates (const QuillImage &image,
					const QPoint &full,
					QPoint &frac) const;
    void fullImageToFractionRect (const QuillImage &image,
				 const QRect &full,
				 QRect &frac) const;

    void getCroppingAndScaling(const QuillImage &image,
			       float &scale, int &cropTop, int &cropLeft) const;


};

#endif
